var m_net = require('net');
var m_events = require('events');
var m_cache;
var m_rest;
var m_utils;

var moduleName = 'app_chromium';
var chromiumUrl;

var chromiumPort = '1233';
var reconnectTimeout = 6000;
var client = new m_net.Socket();
var chromiumData = null;
var cmdResult = null;
var responseEvent = new m_events.EventEmitter();

const status = {
    PENDING: -1,
    OK: 0,
    CMD_ERROR: 1,
    FORMAT_ERROR: 2
}
var cmdStatus = status.PENDING;

function init (modules, cb, jmu_require)
{
    m_cache = jmu_require('../cache');
    m_rest = jmu_require('../api/rest');
    m_utils = jmu_require('../utils');

    m_utils.trace('Initializing Chromium App Module');

    chromiumUrl = m_rest.url + '/' + moduleName;

    client.on('data', function(data) {
        m_utils.trace('Received: ' + data);
        var obj;

        try {
            obj = JSON.parse(data);
        } catch(e) {
            m_utils.error("[Chromium] Received invalid data!");
            return;
        }

        if (obj.data != null)
           chromiumData = obj.data;

        if (obj.status != null) {
            cmdStatus = obj.status;
            cmdResult = obj.result;
            responseEvent.emit('cmdResult');
        }
    });

    client.on('error', function(err) {
        m_utils.trace("[Chromium] Connection to socket failed");
        client.destroy();
    });

    client.on('close', function() {
        setTimeout(function(){
            client.connect(chromiumPort, '127.0.0.1');
        }, reconnectTimeout);
    });

    client.on('connect', function() {
        m_utils.info('[Chromium] Connected to service socket');
    });

    function sendTCP(data, cb) {

        if (!client || client.pending ) {
            if (cb)
                cb({ error : "Connection not ready" });
            return;
        }

        cmdStatus = status.PENDING;
        if (!client.write(data)) {
            stream.once('drain', cb);
        } else if(cb)
            cb(null);
    }

    var $ = this;

    cacheClear();

    $.get = function (cache, cbt)
    {
        m_utils.trace('get() on /' + moduleName + ' - cache: ' + cache);

	if (chromiumData == null) {
            // Data not available
            if(cbt)
                cbt(null, {});
            return;
        }

        if (!m_cache[$.type])
            m_cache[$.type] = {};

	m_cache[$.type].chromium = chromiumData;
        if (cbt)
            cbt(null, chromiumData);
    };

    function routeGet (req, res)
    {
        m_rest.routeGet(req, res, $.get);
    };

    function routePost (req, res)
    {
        m_utils.trace('Received POST on /' + moduleName + ': ' + JSON.stringify(req.body));

        if (client.pending || client.destroyed )
            return res.status(404).json(err);

        var settings = {};
        if ( req.body.homepage && req.body.homepage != "" ) {
            if ( req.body.homepage.indexOf("://") == -1 )
               settings.homepage = "http://" + req.body.homepage;
            else
               settings.homepage = req.body.homepage;
        }

        if ( req.body.onStartup )
            settings.onStartup = req.body.onStartup;

        if ( req.body.enableNavBar != null )
            settings.enableNavBar = req.body.enableNavBar;

        if ( req.body.autoNavBar != null )
            settings.autoNavBar = req.body.autoNavBar;

        if ( req.body.historyBtns != null )
            settings.historyBtns = req.body.historyBtns;

        if ( req.body.loadingBar != null )
            settings.loadingBar = req.body.loadingBar;

        settings.customUserAgent = req.body.customUserAgent;

        if ( req.body.fallbackPage && req.body.fallbackPage.indexOf("://") == -1 )
            settings.fallbackPage = "http://" + req.body.fallbackPage;
        else
            settings.fallbackPage = req.body.fallbackPage;

        if ( req.body.holdOptionsTimer != null ) {
           if (req.body.holdOptionsTimer > 10)
              req.body.holdOptionsTimer = 10;
           if (req.body.holdOptionsTimer < 3)
              req.body.holdOptionsTimer = 3;
           settings.holdOptionsTimerMilliseconds = req.body.holdOptionsTimer *1000;
        }

        if ( req.body.prefs != null && req.body.prefs.services != null ) {
           settings.prefs = { services: {} };
           if ( req.body.prefs.services.passwordSavingEnabled != null )
              settings.prefs.services.passwordSavingEnabled = req.body.prefs.services.passwordSavingEnabled;
           if ( req.body.prefs.services.downloadRestrictions != null )
              settings.prefs.services.downloadRestrictions = req.body.prefs.services.downloadRestrictions;
	}

        responseEvent.removeAllListeners('cmdResult');
        responseEvent.once('cmdResult', function() {
            if ( cmdStatus != status.OK )
                return res.status(404).json({});
            return res.json(cmdResult)
        });

        sendTCP(JSON.stringify({ cmd: "setSettings", settings: settings}), function(err) {
            if (err) {
                m_utils.error('POST /' + moduleName + ': ' + err.message);
                return res.status(404).json(err);
            }
        });
    };

    $.registerHttp = function (app)
    {
        m_utils.info('Registering URL: ' + chromiumUrl);

        app.get(chromiumUrl, routeGet);
        app.post(chromiumUrl, routePost);
    };

    function cacheClear ()
    {
        if ( m_cache[$.type])       
            m_cache[$.type].chromium = null;
    };

    $.registerMonitor = function()
    {
        return { fns : [ $.get ], clear : cacheClear };
    };

    $.type = 'apps';

    client.connect(chromiumPort, '127.0.0.1');

    cb(null);

    return $;
};

exports = module.exports = init;
